/*****************************************************************************\
* (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef DETDESC_SIMPLEVALIDITY_H 
#define DETDESC_SIMPLEVALIDITY_H 1
// Include files
#include "GaudiKernel/IValidity.h"

/** @class SimpleValidity SimpleValidity.h
 *
 *  The simplest implementation of IValidity interface 
 *
 *  @author Vanya Belyaev Ivan.Belyaev@itep.ru
 *  @date   17/11/2001
 */

class SimpleValidity:
  public  virtual  IValidity
{
public:
  
  /** standard constructor
   */
  SimpleValidity(); 
  
  /** standard constructor
   *  @param since "since" time for validity range 
   *  @param till  "till"  time for validity range 
   */
  SimpleValidity( const Gaudi::Time& since    ,
                  const Gaudi::Time& till     ); 
  
  /** (explicit) constructor from other 
   *   IValidity object
   *   @param copy another IValidity object 
   */
  explicit 
  SimpleValidity( const IValidity&      copy );
  
  /** copy constructor  (deep copy)
   *  @param copy another IValidity object 
   */
  SimpleValidity( const SimpleValidity& copy );
  
  /** assignement from any IValidity object 
   *  @param copy another IValidity object 
   */
  SimpleValidity& operator=( const IValidity&      copy );
  
  /** assignement (deep copy)
   *  @param copy another IValidity object 
   */
  SimpleValidity& operator=( const SimpleValidity& copy );
  
  /** destructor 
   */
  virtual ~SimpleValidity();
  
  /** is the Object valid? (it can be always invalid)
   *  @return true if object valid 
   */
  virtual bool 
  isValid    () const; 
  
  /** is the Object valid for a given time?
   *  @param  time  tiemto be checked 
   *  @return true if objetc is valid for given time 
   */
  virtual bool 
  isValid    ( const Gaudi::Time&  time ) const; 
  
  /** since what time the Objest is valid?
   *  @return time "since"
   */
  inline virtual const Gaudi::Time& 
  validSince () const { return m_since; }
  
  /** till  what time the Object is Valid?
   *  @return time "till"
   */
  inline virtual const Gaudi::Time&
  validTill  () const { return m_till ; }
  
  /// set the validity range of the Object
  virtual void 
  setValidity       ( const Gaudi::Time& since , 
                      const Gaudi::Time& till  ) ; 
  /// set the validity time of the Object 
  virtual void
  setValiditySince  ( const Gaudi::Time& since ) ; 
  
  /// set the validity time of the Object 
  virtual void 
  setValidityTill   ( const Gaudi::Time& till  ) ;
  
private:
  
  Gaudi::Time m_since ;
  Gaudi::Time m_till  ;
  
};

// ============================================================================
// The End 
// ============================================================================
#endif // DETDESC_SIMPLEVALIDITY_H
// ============================================================================
