/*****************************************************************************\
* (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#pragma once

#include "GaudiKernel/AlgTool.h"
#include "GaudiKernel/IEventTimeDecoder.h"
#include "GaudiKernel/IIncidentSvc.h"

/** Basic implementation of an EventTimeDecoder.
 *  It just provides fake event times.
 *
 *  @author Marco Clemencic
 *  @date   2006-09-21
 */
class FakeEventTime : public extends1<AlgTool, IEventTimeDecoder> {
public:
  /// Standard constructor
  FakeEventTime( const std::string& type,
                 const std::string& name,
                 const IInterface* parent);

  /// Initialization
  StatusCode initialize() override;
  StatusCode start() override;
  StatusCode stop() override;

  // --- implementation of IEventTimeDecoder ---

  /// Return the time of current event.
  Gaudi::Time getTime() const override;

private:

  // ---------- data members ----------

  /// First event time (when simluating them).
  /// Set by the option StartTime (default = 0)
  long long m_startTime;
  /// Event time increment (when simluating event times).
  /// Set by the option TimeStep (default = 0, which means no simulation)
  long long m_timeStep;

  unsigned long m_startRun;
  unsigned long m_eventsPerRun;


  unsigned long m_evtCount = 0;

  SmartIF<IIncidentSvc> m_incSvc;

  /// internal method to increment the state
  void i_increment();
};
