/*****************************************************************************\
* (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
// Include files 
#include <utility>
#include <string>
#include <map>
#include <sstream>

#include "Event/L0DUElementaryData.h"

//-----------------------------------------------------------------------------
// Auxilliary Implementation file for L0DUElementaryData Event classes
//
// 09/02/2006 : Olivier Deschamps
//-----------------------------------------------------------------------------

unsigned int LHCb::L0DUElementaryData::operation(const std::string& v_operator, const std::vector<unsigned int>& 
v_operands){
  unsigned int digit = 0;
  // 1 dimensional operator (Identity)
  if(1 == v_operands.size() ){
    if( "Id" == v_operator ){digit = v_operands[0] ;}
  }
  // 2 dimensional operator (+,-)
  if(2 == v_operands.size() ){
    if( "+" == v_operator ){digit = v_operands[0] + v_operands[1] ;}
    if( "-" == v_operator ){
	digit = ( v_operands[0] > v_operands[1]) ? v_operands[0] - v_operands[1] : 0;
    } 
    if( "&" == v_operator ){digit = v_operands[0] & v_operands[1] ;}
    if( "^" == v_operator ){digit = v_operands[0] ^ v_operands[1] ;}
  }
  // Insert here other N-dimensional operators you need (LUT,...)
  // ...
  //  
  return digit;
}


std::string LHCb::L0DUElementaryData::summary(){
  std::ostringstream s(" ");
  std::ostringstream ss(" ");
  if(m_operator != "Id")
    ss << " = op[" << m_operator << "](" << operandsName() << ")";
  std::ostringstream sss(" ");
  if(scale() != 1.)sss<< "  ( => " << value() << " MeV) " ;
  s << " [ " << m_name << ss.str() << "  : " <<  digit() << sss.str() << " ] ";
  return s.str();
}


std::string LHCb::L0DUElementaryData::description(){
  std::ostringstream s(" ");
  std::ostringstream ss(" ");
  if(m_operator != "Id")ss << " = [" << m_operator << "](" << m_operandsName << ")" ;
  s << "[ " << m_name << " " << ss.str() << "] ";
  return s.str();

}

