###############################################################################
# (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
from GaudiTest import normalizeExamples, RegexpReplacer, LineSkipper

gitCondDBFixes = (RegexpReplacer(when='Detector description database:',
                                 orig='conddb:/', repl='git:/') +
                  LineSkipper(['CORAL Connection Retrial',
                               'INFO Connected to database',
                               'INFO opening Git repository',
                               'INFO using commit']))

preprocessor = (
    normalizeExamples +
    RegexpReplacer(  # normalize full path to DBASE or PARAM
        orig = r'/[^ :]+/(DBASE|PARAM)/',
        repl = r'\1/') +
    RegexpReplacer(  # hide release directories
        when = "cern.ch",
        orig = r'/afs/cern.ch/lhcb/software/(DEV/nightlies|releases)/|/cvmfs/lhcb.cern.ch/lib/lhcb',
        repl = r'') +
    RegexpReplacer(  # hide data package versions
        orig = r'((DBASE|PARAM)/([^/]+/)?[^/]+)/v[0-9]+r[0-9]+(p[0-9]+)?/',
        repl = r'\1/vXrYpZ/') +
    RegexpReplacer(when = "at 0x########L",
        orig = r'0x########L',
        repl = r'0x########') + \
    RegexpReplacer(when = "Connected to database",
        orig = r'ONLINE[_-][0-9 ]{1,6}(.*)"[^"]*/([0-9A-Z_]{1,8})"',
        repl = r'ONLINE_xxxxxx\1"\2"') + \
    RegexpReplacer(when = "Connected to database",
        orig = r'"[^"]*/([0-9A-Z_]{1,8})"',
        repl = r'"\1"') +
    LineSkipper(["EventSelector        INFO Stream:EventSelector.DataStreamTool",
                 "INFO Using TAG", "TimingAuditor.T", "RootDBase.open",
                 "INFO Opened magnetic field file",
                 "INFO Connecting to database",
                 "INFO Disconnect from database after being idle",
                 "INFO Memory has changed from",
                 "INFO Memory change after pool release",
                 "Memory for the event exceeds 3*sigma",
                 "Mean 'delta-memory' exceeds 3*sigma",
                 "INFO  'CnvServices':",
                 "DEBUG Property ['Name': Value] =  'ErrorCount",
                 "DEBUG Property ['Name': Value] =  'RootOnTES':",
                 "VERBOSE ServiceLocatorHelper::service: found service",
                 "DEBUG Property ['Name': Value] =  'NeededResources':[  ]",
                 "DEBUG Property ['Name': Value] =  'Cardinality':1",
                 "DEBUG Property ['Name': Value] =  'IsClonable':False",
                 "#properties =",
                 "VERBOSE ServiceLocatorHelper::service: found service AlgExecStateSvc",
                 "Run numbers generated from 0 every 0 events",
                ],
      regexps = [r"DEBUG Property \['Name': Value\] =  '(Timeline|(Extra|Data)(In|Out)puts)'",
                ])
   )

from DDDB.Configuration import GIT_CONDDBS
if GIT_CONDDBS:
    preprocessor = preprocessor + gitCondDBFixes
