/*****************************************************************************\
* (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#include "FileReadTool.h"
#include <fstream>

DECLARE_COMPONENT( FileReadTool )

FileReadTool::FileReadTool( const std::string& type,
                            const std::string& name,
                            const IInterface* parent):
  base_class(type, name, parent)
{
  //declareInterface<IFileAccess>(this);
}

std::unique_ptr<std::istream> FileReadTool::open(const std::string &url) {
  // remove the optional "file://" from the beginning of the url
  std::string path;
  if ( url.compare(0,7,"file://") == 0 ) {
    path = url.substr(7);
  } else {
    path = url;
  }
  return std::unique_ptr<std::istream>(new std::ifstream{path});
}

const std::vector<std::string> &FileReadTool::protocols() const
{
  /// Vector of supported protocols.
  static const std::vector<std::string> s_protocols = {{"file"}};
  return s_protocols;
}
