/*****************************************************************************\
* (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef _FILEREADTOOL_H
#define _FILEREADTOOL_H

#include "GaudiKernel/IFileAccess.h"
#include "GaudiKernel/AlgTool.h"

/** @class FileReadTool FileReadTool.h
 *
 *  Basic implementation of the IFileAccess interface.
 *  This tool simply takes a path to a file as url and return the std::istream interface
 *  of std::ifstream.
 *
 *  @author Marco Clemencic
 *  @date 2008-01-18
 */
class FileReadTool : public extends<AlgTool,
                                    IFileAccess> {

public:

  /// Standard constructor
  FileReadTool( const std::string& type,
                const std::string& name,
                const IInterface* parent);

  ~FileReadTool() override = default;

  std::unique_ptr<std::istream> open(const std::string &url) override;

  /// Protocols supported by the instance.
  const std::vector<std::string> &protocols() const override;

};

#endif  // _FILEREADTOOL_H
