/*****************************************************************************\
* (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef SRC_SERVICESTARTER_H
#define SRC_SERVICESTARTER_H 1
// Include files
// from Gaudi
#include "GaudiAlg/GaudiAlgorithm.h"

namespace LHCbAlgsTest {

/** @class ServiceStarter ServiceStarter.h src/ServiceStarter.h
 *
 * Simple test algorithm to trigger the instantiation of a service in
 * a specific phase (initialize, start or execute).
 *
 * @author Marco Clemencic
 * @date 09/01/2012
 */
class ServiceStarter: public GaudiAlgorithm {
public:
  /// Standard constructor
  ServiceStarter(const std::string& name, ISvcLocator* pSvcLocator);
  virtual ~ServiceStarter(); ///< Destructor

  virtual StatusCode initialize();    ///< Algorithm initialization
  virtual StatusCode start     ();    ///< Algorithm start
  virtual StatusCode execute   ();    ///< Algorithm execution
  virtual StatusCode finalize  ();    ///< Algorithm finalization
protected:
private:
  /// Service to get.
  std::string m_serviceName;

  /// When to get the service;
  std::string m_phase;

  /// Internal reference to the retrieved service.
  SmartIF<IService> m_service;

  StatusCode i_retrieveService(const std::string &currentPhase);
};

}
#endif // SRC_SERVICESTARTER_H
