/*****************************************************************************\
* (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef L0DUMULTICONFIGPROVIDER_H 
#define L0DUMULTICONFIGPROVIDER_H 1

// Include files
// from Gaudi
#include "GaudiAlg/GaudiTool.h"
// from LHCb
#include "L0Interfaces/IL0DUConfigProvider.h"            // Interface
#include "L0DUConfigProvider.h" // single Config provider tool


/** @class L0DUMultiConfigProvider L0DUMultiConfigProvider.h
 *  
 *  Provide  **ANY** predefined L0DU configuration according to 
 *  the Trigger Configuration Key
 *  Configuration described in options file
 *
 *  @author Olivier Deschamps
 *  @date   2008-01-26
 */
class L0DUMultiConfigProvider : public GaudiTool, virtual public IL0DUConfigProvider {
public: 
  /// Standard constructor
  L0DUMultiConfigProvider( const std::string& type, 
                           const std::string& name,
                           const IInterface* parent);

  virtual ~L0DUMultiConfigProvider( ); ///< Destructor

  virtual StatusCode initialize();
  virtual StatusCode finalize();

  LHCb::L0DUConfig*  config(long tck = LHCb::L0DUTemplateConfig::TCKValue,std::string slot="T0");
  LHCb::L0DUConfigs*  configs(std::string slot="T0"){return m_configs[slot];};

  
protected:

private:
  // private method
  LHCb::L0DUConfig* loadConfig( std::string tck ,std::string slot="T0");

  // attributes
  bool m_preload;
  std::vector<std::string> m_list;

  //
  std::map<std::string,LHCb::L0DUConfigs*> m_configs;
  IL0DUConfigProvider* m_provider;
  std::map<std::string,LHCb::L0DUConfig*> m_template;

};
#endif // L0DUMULTICONFIGPROVIDER_H
