/*****************************************************************************\
* (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
// ============================================================================
// Include files 
// ============================================================================
// GaudiKernel
// ============================================================================
// GaudiKernel
// ============================================================================
#include "GaudiKernel/Kernel.h"
#include "GaudiKernel/MsgStream.h"
#include "GaudiKernel/IMessageSvc.h"
// ============================================================================
// LoKi
// ============================================================================
#include "LoKi/Colors.h"
// ============================================================================
/** @file
 *
 *  Helper file with functions to provide colorized output 
 * 
 *  This file is a part of LoKi project - 
 *    "C++ ToolKit  for Smart and Friendly Physics Analysis"
 *
 *  The package has been designed with the kind help from
 *  Galina PAKHLOVA and Sergey BARSUK.  Many bright ideas, 
 *  contributions and advices from G.Raven, J.van Tilburg, 
 *  A.Golutvin, P.Koppenburg have been used in the design.
 *
 *  @author Vanya Belyaev Ivan.Belyaev@cern.ch
 *  @date   2011-06-03
 */
// ============================================================================
/*  set the colors for output stream
 *  @param  stream  reference for stream
 *  @param  fg      color for foreground
 *  @param  fg      color for foreground
 *  @return true if color are changed
 */
// ============================================================================
bool LoKi::Colors::setColor( std::ostream& stream, const MSG::Color& fg, const MSG::Color& bg ) {
  int fc = 30 + fg;
  int bc = 40 + bg;
  stream << "\033[" << bc << "m\033[1;" << fc << "m";
  return true;
}
// ============================================================================
/* reset the colors for output stream
 *  @param  stream  reference for stream
 *  @return true if color are changed
 */
bool LoKi::Colors::resetColor( std::ostream& stream ) {
  stream << "\033[0m";
  return true;
}
// ============================================================================
/* reset the colors for output stream
 *  @param  stream  reference for stream
 *  @return true if color are changed
 */
// ============================================================================
bool LoKi::Colors::resetColor ( MsgStream&        stream ) 
{
  stream.resetColor() ;
  return true ;
}
// ============================================================================
/*  set the colors for output stream 
 *  @param  stream  reference for stream 
 *  @param  fg      color for foreground 
 *  @param  fg      color for foreground 
 *  @return true if color are changed 
 */
// ============================================================================
bool LoKi::Colors::setColor 
( MsgStream&        stream ,
  const MSG::Color& fg     , 
  const MSG::Color& bg     )
{
  stream.setColor ( fg , bg );
  return true ;
}
// ============================================================================


// ============================================================================
// The END 
// ============================================================================
