/*****************************************************************************\
* (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
// Include files 

// from LHCb
#include "Event/MCParticle.h"
#include "Event/MCVertex.h"

// local
#include "FlagSignalChain.h"

//-----------------------------------------------------------------------------
// Implementation file for class : FlagSignalChain
//
// 2015-07-23 : Gloria Corti
//-----------------------------------------------------------------------------

// Declaration of the Tool Factory
DECLARE_TOOL_FACTORY( FlagSignalChain )


//=============================================================================
// Standard constructor, initializes variables
//=============================================================================
FlagSignalChain::FlagSignalChain( const std::string& type,
                                  const std::string& name,
                                  const IInterface* parent )
  : GaudiTool ( type, name , parent )
{
  declareInterface<IFlagSignalChain>(this);

}
//=============================================================================
// Destructor
//=============================================================================
FlagSignalChain::~FlagSignalChain() {} 

//==============================================================================
// Set from signal flag
//==============================================================================
void FlagSignalChain::setFromSignalFlag( const LHCb::MCParticle* mother ) {

  SmartRefVector<LHCb::MCVertex>::const_iterator iv;
  for ( iv = mother->endVertices().begin();
        iv != mother->endVertices().end(); iv++ ) {
    SmartRefVector<LHCb::MCParticle>::const_iterator idau;
    for ( idau = (*iv)->products().begin();
          idau != (*iv)->products().end(); idau++ ) {
      const LHCb::MCParticle* mcpc = *idau;
      LHCb::MCParticle* mcp = const_cast< LHCb::MCParticle* >( mcpc ) ;
      mcp->setFromSignal(true);
      setFromSignalFlag( *idau );
    }
  }
  
}
//==============================================================================
