/*****************************************************************************\
 * (c) Copyright 2018 CERN for the benefit of the LHCb Collaboration           *
 *                                                                             *
 * This software is distributed under the terms of the GNU General Public      *
 * Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
 *                                                                             *
 * In applying this licence, CERN does not waive the privileges and immunities *
 * granted to it by virtue of its status as an Intergovernmental Organization  *
 * or submit itself to any jurisdiction.                                       *
\*****************************************************************************/

#ifndef GENFSRJSON_H
#define GENFSRJSON_H 1

// Include files
// From Gaudi
#include "GaudiAlg/GaudiAlgorithm.h"
#include "GaudiKernel/IDataProviderSvc.h"
#include "GaudiKernel/AlgFactory.h"

// from Event
#include "Event/GenFSR.h"

// from Kernel
#include "FSRAlgs/IFSRNavigator.h"

/** @class GenFSRJson GenFSRJson.h
 *
 *
 *  @author Davide Fazzini
 *  @date   2018-06-26
 */

class GenFSRJson : public GaudiAlgorithm {
public:
  /// Standard constructor
  GenFSRJson( const std::string& name, ISvcLocator* pSvcLocator );

  virtual StatusCode initialize(); ///< Algorithm initialization
  virtual StatusCode execute();    ///< Algorithm execution
  virtual StatusCode finalize();   ///< Algorithm finalization

  void printFSR(); // Print the GenFSR in a file .json

private:
  std::string m_fileRecordName;        // TES location where FSRs are persisted
  std::string m_FSRName;               // Name of the genFSR tree
  std::string m_prodID;                // Production ID used in the generation
  std::string m_appConfigVersion;      // Version of AppConfig used in the simulation
  std::string m_appConfigFile;         // Name of the AppConfig file
  std::string m_gaussVersion;          // Gauss version used in the simulation
  std::string m_simCond;               // Tag for the SimCond database
  std::string m_dddb;                  // Tag for the DDDB database
  std::string m_jsonOutputLocation;    // Path where to save the .json output
  std::string m_jsonOutputName;        // Name of the .json output

  IDataProviderSvc* m_fileRecordSvc;
  IFSRNavigator*    m_navigatorTool;   // Tool to navigate FSR
};
#endif // GENFSRJSON_H
