/*****************************************************************************\
* (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef GENFSRREAD_H
#define GENFSRREAD_H 1

// Include files
// from Gaudi
#include "GaudiAlg/GaudiAlgorithm.h"
#include "GaudiKernel/IDataProviderSvc.h"
#include "GaudiKernel/AlgFactory.h"

// from Event
#include "Event/GenFSR.h"

// from Kernel
#include "FSRAlgs/IFSRNavigator.h"

/** @class GenFSRRead GenFSRRead.h
 *
 *
 *  @author Davide Fazzini
 *  @date   2015-06-23
 */

class GenFSRRead : public GaudiAlgorithm {
public:
  // Standard constructor
  GenFSRRead( const std::string& name, ISvcLocator* pSvcLocator );

  virtual StatusCode initialize(); // Algorithm initialization
  virtual StatusCode execute();    // Algorithm execution
  virtual StatusCode finalize();   // Algorithm finalization

  void printFSR(); // Print the GenFSR

private:
  std::string m_fileRecordName;      // TES location where FSRs are persisted
  std::string m_FSRName;             // Name of the genFSR tree

  IDataProviderSvc* m_fileRecordSvc;
  IFSRNavigator*    m_navigatorTool; // Tool to navigate FSR
};
#endif // GENFSRREAD_H
